@@ -1,9 +1,13 @@
 from pytestarch.eval_structure.evaluable_architecture import (  # noqa: F401
     EvaluableArchitecture,
 )
+from pytestarch.query_language.rule import Rule  # noqa: F401
 from .pytestarch import get_evaluable_architecture  # noqa: F401 E401
 from .pytestarch import (  # noqa: F401 E401; noqa: F401 E401
     get_evaluable_architecture_for_module_objects,
 )
-from pytestarch.query_language.rule import Rule  # noqa: F401
+from pytestarch.query_language.layered_architecture_rule import (  # noqa: F401
+    LayeredArchitecture,
+    LayerRule,
+)
 from pytestarch.diagram_extension.diagram_rule import DiagramRule  # noqa: F401
