@@ -1,10 +1,9 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
-from typing import Dict, Set
 
 
 @dataclass(frozen=True)
 class ParsedDependencies:
-    all_modules: Set[str]
-    dependencies: Dict[str, Set[str]]
+    all_modules: set[str]
+    dependencies: dict[str, set[str]]
