@@ -1,19 +1,17 @@
 from __future__ import annotations
 
-from typing import List, Set
-
-from pytestarch.eval_structure.evaluable_architecture import Module, StrictDependency
+from pytestarch.eval_structure.evaluable_architecture import Dependency, ModuleFilter
 from pytestarch.eval_structure.evaluable_structures import AbstractGraph, AbstractNode
-from pytestarch.eval_structure.utils import get_excluded_nodes, get_node, to_modules
+from pytestarch.eval_structure.utils import get_node, get_parent_nodes, to_modules
 
 
 def get_dependency_between_modules(
-    graph: AbstractGraph, dependent: Module, dependent_upon: Module
-) -> List[StrictDependency]:
+    graph: AbstractGraph, dependent: ModuleFilter, dependent_upon: ModuleFilter
+) -> list[Dependency]:
     dependent_node = get_node(dependent)
     dependent_upon_nodes = get_all_submodules_of(graph, dependent_upon)
 
-    nodes_to_exclude = get_excluded_nodes([dependent, dependent_upon])
+    nodes_to_exclude = get_parent_nodes([dependent, dependent_upon])
 
     nodes_to_check = [dependent_node]
     checked_nodes = set()
@@ -44,8 +42,8 @@ def get_dependency_between_modules(
 
 
 def any_dependency_to_module_other_than(  # noqa: C901
-    graph: AbstractGraph, dependent: Module, dependent_upons: Set[Module]
-) -> List[StrictDependency]:
+    graph: AbstractGraph, dependent: ModuleFilter, dependent_upons: set[ModuleFilter]
+) -> list[Dependency]:
     # nodes to exclude are nodes that once reached will not have their submodules analysed next
     nodes_to_exclude = set()
     for dependent_upon in dependent_upons:
@@ -64,20 +62,20 @@ def any_dependency_to_module_other_than(  # noqa: C901
     # should submodules of the dependent module import each other, this does not count as a dependency
     nodes_that_do_not_fulfill_criterion = get_all_submodules_of(graph, dependent)
 
-    if dependent.parent_module is not None:
+    if dependent.identifier_is_parent_module:
         # if the parent module is set and has a dependency other than dependent upon, it should not count as only
         # the dependent and its submodules should be considered
         # Example: if we are looking for imports by A.X, we do not care if A itself imports something
         # (but we do care if A.X.M (submodule of A.X) imports something, as this is part of A.X)
-        nodes_to_exclude.add(dependent.parent_module)
+        nodes_to_exclude.add(dependent.identifier)
 
     for dependent_upon in dependent_upons:
-        if dependent_upon.parent_module is not None:
+        if dependent_upon.identifier_is_parent_module:
             # if there is a dependency to the parent module of dependent upon, this counts, as only dependencies to
             # the true submodules are excluded
             # reason: if something imports B, then it imports something that is not B.X (and B.X is a dependent_upon module)
             # (but we do not care if it imports B.X.M, as this is part of B.X)
-            nodes_to_exclude.remove(dependent_upon.parent_module)
+            nodes_to_exclude.remove(dependent_upon.identifier)
 
     nodes_to_check = list(nodes_that_do_not_fulfill_criterion)
     checked_nodes = set()
@@ -109,8 +107,8 @@ def any_dependency_to_module_other_than(  # noqa: C901
 
 
 def any_other_dependency_to_module_than(  # noqa: C901
-    graph: AbstractGraph, dependents: Set[Module], dependent_upon: Module
-) -> List[StrictDependency]:
+    graph: AbstractGraph, dependents: set[ModuleFilter], dependent_upon: ModuleFilter
+) -> list[Dependency]:
     # submodules of the dependent upon module do not count as an import that is not the dependent upon module
     # submodules of and including the dependent do not count as allowed imports
 
@@ -134,22 +132,20 @@ def any_other_dependency_to_module_than(  # noqa: C901
         graph, dependent_upon
     )
 
-    if dependent_upon.parent_module is not None:
+    if dependent_upon.identifier_is_parent_module:
         # if the dependent upon module is defined via a parent module, this parent module is not included in the
         # list of modules that do not fulfill the criteria - only its true submodules are
         # reason: there is a dependency from the parent module to the child module, but this is not an import
-        nodes_that_count_as_not_fulfilling_criterion.remove(
-            dependent_upon.parent_module
-        )
+        nodes_that_count_as_not_fulfilling_criterion.remove(dependent_upon.identifier)
 
     nodes_to_check = list(nodes_that_count_as_not_fulfilling_criterion)
 
     for dependent in dependents:
-        if dependent.parent_module is not None:
+        if dependent.identifier_is_parent_module:
             # if the dependent module is defined via a parent module, this parent module counts as an allowed import
             # reason: we are looking for dependencies other than module A.X. Parent module A is not (only) A.X, so it counts
             # (submodule A.X.M does not count, as it is not anything else but A.X)
-            nodes_to_exclude.remove(dependent.parent_module)
+            nodes_to_exclude.remove(dependent.identifier)
 
     checked_nodes = set()
 
@@ -177,7 +173,9 @@ def any_other_dependency_to_module_than(  # noqa: C901
     return nodes_fulfilling_criteria  # type: ignore
 
 
-def get_all_submodules_of(graph: AbstractGraph, module: Module) -> Set[AbstractNode]:
+def get_all_submodules_of(
+    graph: AbstractGraph, module: ModuleFilter
+) -> set[AbstractNode]:
     """Returns all submodules of a given module.
 
     Args:
