@@ -1,14 +1,13 @@
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
-from typing import List
 
 AbstractNode = str
 
 
 class AbstractGraph(ABC):
     @abstractmethod
-    def direct_successor_nodes(self, node: AbstractNode) -> List[AbstractNode]:
+    def direct_successor_nodes(self, node: AbstractNode) -> list[AbstractNode]:
         raise NotImplementedError()
 
     @abstractmethod
@@ -18,10 +17,10 @@ class AbstractGraph(ABC):
         raise NotImplementedError()
 
     @abstractmethod
-    def direct_predecessor_nodes(self, node: AbstractNode) -> List[AbstractNode]:
+    def direct_predecessor_nodes(self, node: AbstractNode) -> list[AbstractNode]:
         raise NotImplementedError()
 
     @property
     @abstractmethod
-    def nodes(self) -> List[AbstractNode]:
+    def nodes(self) -> list[AbstractNode]:
         raise NotImplementedError()
