@@ -1,8 +1,10 @@
 """Encapsulation of networkx graph functionality."""
+
 from __future__ import annotations
 
 import re
-from typing import Any, Dict, List, Optional, Tuple
+from collections.abc import Sequence
+from typing import Any
 
 import networkx as nx
 from networkx import draw_networkx, has_path, spring_layout
@@ -32,9 +34,9 @@ class NetworkxGraph(AbstractGraph):
 
     def __init__(
         self,
-        all_modules: List[Node],
-        imports: List[Import],
-        level_limit: Optional[int] = None,
+        all_modules: list[Node],
+        imports: Sequence[Import],
+        level_limit: int | None = None,
     ) -> None:
         """
         Args:
@@ -84,7 +86,7 @@ class NetworkxGraph(AbstractGraph):
 
     def _add_edges_within_module_hierarchy(
         self,
-        parent_modules: List[Node],
+        parent_modules: list[Node],
         child: Node,
     ) -> None:
         """Create edges between a node and its parent recursively until the parent-less parent is reached.
@@ -162,14 +164,14 @@ class NetworkxGraph(AbstractGraph):
         return self._graph.number_of_nodes()
 
     @property
-    def nodes(self) -> List[Node]:
+    def nodes(self) -> list[Node]:
         return list(self._graph.nodes)
 
     @property
-    def edges(self) -> List[Tuple[Node, Node]]:
+    def edges(self) -> list[tuple[Node, Node]]:
         return list(self._graph.edges)
 
-    def direct_predecessor_nodes(self, node: Node) -> List[Node]:
+    def direct_predecessor_nodes(self, node: Node) -> list[Node]:
         """Returns all nodes that have a directed edge towards the given node.
 
         Args:
@@ -180,7 +182,7 @@ class NetworkxGraph(AbstractGraph):
         """
         return sorted(self._graph.predecessors(node))
 
-    def direct_successor_nodes(self, node: Node) -> List[Node]:
+    def direct_successor_nodes(self, node: Node) -> list[Node]:
         """Returns all nodes that the given node has a directed edge towards.
 
         Args:
@@ -225,6 +227,13 @@ class NetworkxGraph(AbstractGraph):
             Remaining keyword args are passed to draw_networkx. Notably, "ax" can be
             used to draw the graph onto an existing matploblib Axis object.
         """
+        try:
+            import matplotlib  # noqa: F401
+        except ImportError:
+            raise Exception(
+                "Graph visualisations require matplotlib. Please install it, via pip install pytestarch[visualization]."
+            )
+
         if "spacing" in kwargs:
             spacing = kwargs.pop("spacing")
             pos = spring_layout(self._graph, k=spacing, iterations=20)
@@ -237,8 +246,8 @@ class NetworkxGraph(AbstractGraph):
 
         draw_networkx(self._graph, **kwargs)
 
-    def _create_plot_labels_with_alias(self, aliases: Dict[str, str]) -> Dict[str, str]:
-        module_names: List[str] = list(self._graph.nodes)
+    def _create_plot_labels_with_alias(self, aliases: dict[str, str]) -> dict[str, str]:
+        module_names: list[str] = list(self._graph.nodes)
         self._assert_aliased_modules_exist(aliases, module_names)
 
         # longest name first so aliases for submodule take priority over aliases  for
@@ -257,8 +266,8 @@ class NetworkxGraph(AbstractGraph):
 
     def _assert_aliased_modules_exist(
         self,
-        aliases: Dict[str, str],
-        module_names: List[str],
+        aliases: dict[str, str],
+        module_names: list[str],
     ) -> None:
         for module in aliases:
             if module not in module_names:
@@ -270,8 +279,8 @@ class NetworkxGraph(AbstractGraph):
     def _create_label(
         self,
         module_name: str,
-        sorted_aliased_modules: List[str],
-        aliases: Dict[str, str],
+        sorted_aliased_modules: list[str],
+        aliases: dict[str, str],
     ) -> str:
         try:
             most_specific_aliased_module = next(
