@@ -1,7 +1,7 @@
 from __future__ import annotations
 
+from collections.abc import Sequence
 from pathlib import Path
-from typing import List, Set
 
 from pytestarch.eval_structure.types import Import
 
@@ -14,9 +14,9 @@ class ImporteeModuleCalculator:
 
     def calculate_importee_modules(
         self,
-        imports: List[Import],
-        all_modules: List[str],
-    ) -> List[str]:
+        imports: Sequence[Import],
+        all_modules: list[str],
+    ) -> list[str]:
         """For all imported modules: Calculate parent modules and add them to the list of existing modules if they
         are not already part of this list. This mainly applies to external dependencies.
 
@@ -37,7 +37,7 @@ class ImporteeModuleCalculator:
 
         return list(extended_modules)
 
-    def _calculate_parent_modules(self, imp: Import) -> Set[str]:
+    def _calculate_parent_modules(self, imp: Import) -> set[str]:
         modules = {imp.importee()}
         modules.update(imp.importee_parent_modules())
 
