@@ -2,12 +2,12 @@
 The following two functions are the main entry point to PyTestArch. They can be used to create an evaluable object,
 for which the user can then define architectural rules.
 """
+
 from __future__ import annotations
 
 import os
 from pathlib import Path
 from types import ModuleType
-from typing import Optional, Tuple
 
 from pytestarch import EvaluableArchitecture
 from pytestarch.eval_structure_generation.graph_generation.graph_generator import (
@@ -24,10 +24,10 @@ DEFAULT_EXCLUSIONS = ("*__pycache__*",)
 def get_evaluable_architecture(
     root_path: str,
     module_path: str,
-    exclusions: Tuple[str, ...] = DEFAULT_EXCLUSIONS,
+    exclusions: tuple[str, ...] = DEFAULT_EXCLUSIONS,
     exclude_external_libraries: bool = True,
-    level_limit: Optional[int] = None,
-    regex_exclusions: Optional[Tuple[str, ...]] = None,
+    level_limit: int | None = None,
+    regex_exclusions: tuple[str, ...] | None = None,
 ) -> EvaluableArchitecture:
     """Constructs an evaluable object based on the given module.
 
@@ -50,22 +50,22 @@ def get_evaluable_architecture(
         )
 
     if exclusions:
-        regex_exclusions = [
+        regex_exclusions = tuple(
             convert_partial_match_to_regex(pattern) for pattern in exclusions
-        ]
+        )
 
-    root_path = Path(root_path)
-    module_path = Path(module_path)
+    root_as_path = Path(root_path)
+    module_as_path = Path(module_path)
 
-    path_diff_between_root_and_module = str(module_path.relative_to(root_path)).replace(
-        os.sep, "."
-    )
+    path_diff_between_root_and_module = str(
+        module_as_path.relative_to(root_as_path)
+    ).replace(os.sep, ".")
 
     return generate_graph(
-        root_path,
-        module_path,
+        root_as_path,
+        module_as_path,
         path_diff_between_root_and_module,
-        regex_exclusions,
+        regex_exclusions,  # type: ignore
         exclude_external_libraries,
         level_limit,
     )
@@ -74,16 +74,16 @@ def get_evaluable_architecture(
 def get_evaluable_architecture_for_module_objects(
     root_module: ModuleType,
     module: ModuleType,
-    exclusions: Tuple[str, ...] = DEFAULT_EXCLUSIONS,
+    exclusions: tuple[str, ...] = DEFAULT_EXCLUSIONS,
     exclude_external_libraries: bool = True,
-    level_limit: Optional[int] = None,
-    regex_exclusions: Optional[Tuple[str, ...]] = None,
+    level_limit: int | None = None,
+    regex_exclusions: tuple[str, ...] | None = None,
 ) -> EvaluableArchitecture:
     """Same functionality as get_evaluable_architecture, but root module and module to evaluate are passed in as module objects
     instead of the absolute paths to them.
     """
-    root_path = os.path.dirname(root_module.__file__)
-    module_path = os.path.dirname(module.__file__)
+    root_path: str = os.path.dirname(root_module.__file__)  # type: ignore
+    module_path: str = os.path.dirname(module.__file__)  # type: ignore
 
     return get_evaluable_architecture(
         root_path,
