@@ -2,7 +2,7 @@ from __future__ import annotations
 
 from abc import ABC, abstractmethod
 from pathlib import Path
-from typing import Generic, List, TypeVar, Union
+from typing import Generic, TypeVar, Union
 
 from pytestarch import EvaluableArchitecture
 from pytestarch.utils.decorators import deprecated
@@ -79,7 +79,7 @@ class BehaviorSpecification(BehaviorBaseSpecification["DependencySpecification"]
 
 
 ModuleSpecificationSuccessor = TypeVar(
-    "ModuleSpecificationSuccessor", BehaviorBaseSpecification, RuleApplier
+    "ModuleSpecificationSuccessor", BehaviorSpecification, RuleApplier
 )
 
 
@@ -89,20 +89,20 @@ class ModuleSpecification(Generic[ModuleSpecificationSuccessor], ABC):
 
     @abstractmethod
     def are_sub_modules_of(
-        self, modules: Union[str, List[str]]
+        self, modules: str | list[str]
     ) -> ModuleSpecificationSuccessor:
         """If multiple rule subjects are specified, this has the same effect as defining a rule per rule subject."""
         pass
 
     @abstractmethod
-    def are_named(self, names: Union[str, List[str]]) -> ModuleSpecificationSuccessor:
+    def are_named(self, names: str | list[str]) -> ModuleSpecificationSuccessor:
         """If multiple rule subjects are specified, this has the same effect as defining a rule per rule subject."""
         pass
 
     @deprecated
     @abstractmethod
     def have_name_containing(
-        self, partial_name: Union[str, List[str]]
+        self, partial_name: str | list[str]
     ) -> ModuleSpecificationSuccessor:
         """
         [DEPRECATED] Use have_name_matching instead; method will be removed in upcoming releases.
@@ -115,7 +115,10 @@ class ModuleSpecification(Generic[ModuleSpecificationSuccessor], ABC):
         self,
         regex: str,
     ) -> ModuleSpecificationSuccessor:
-        """If multiple rule subjects are specified, this has the same effect as defining a rule per rule subject."""
+        """Note that regex expressions can have unexpected results if predicates apply to modules, but not their submodules.
+        This is often the case for regex expressions with negations. Refer to the documentation of the module import feature for more details.
+        If multiple rule subjects are specified, this has the same effect as defining a rule per rule subject.
+        """
         pass
 
 
@@ -174,9 +177,7 @@ class LayerDefinition(ABC):
     or multiple modules."""
 
     @abstractmethod
-    def containing_modules(
-        self, modules: Union[str, List[str]]
-    ) -> Union[LayerName, BaseLayeredArchitecture]:
+    def containing_modules(self, modules: str | list[str]) -> BaseLayeredArchitecture:
         """If a module is defined as belonging to layer X, then its submodules are also assumed to be part of layer X."""
         pass
 
@@ -184,9 +185,11 @@ class LayerDefinition(ABC):
     def have_modules_with_names_matching(
         self,
         regex: str,
-    ) -> Union[LayerName, BaseLayeredArchitecture]:
+    ) -> BaseLayeredArchitecture:
         """If a module is defined as belonging to layer X, then its submodules are also assumed to be part of layer X.
         Note that no attempt will be made to ensure that the regex patterns for different layers are mutually exclusive.
+        Also note that regex expressions can have unexpected results if predicates apply to modules, but not their submodules.
+        This is often the case for regex expressions with negations. Refer to the documentation of the module import feature for more details.
         """
         pass
 
@@ -200,7 +203,11 @@ class LayerName(ABC):
 
 
 class BaseLayeredArchitecture(ABC):
-    pass
+
+    @abstractmethod
+    def with_layer(self) -> LayerName:
+        """This is simply a convenience method to achieve proper typing. It can be omitted."""
+        pass
 
 
 class AccessSpecification(RelationshipSpecification, ABC):
@@ -241,7 +248,7 @@ LayerSpecificationSuccessor = TypeVar(
     "LayerSpecificationSuccessor", LayerBehaviorSpecification, RuleApplier
 )
 
-InputTypes = TypeVar("InputTypes", str, Union[str, List[str]])
+InputTypes = TypeVar("InputTypes", str, Union[str, list[str]])
 
 
 class LayerSpecification(Generic[LayerSpecificationSuccessor, InputTypes], ABC):
@@ -252,7 +259,7 @@ class LayerSpecification(Generic[LayerSpecificationSuccessor, InputTypes], ABC):
         pass
 
 
-class LayerRuleObject(LayerSpecification[RuleApplier, Union[str, List[str]]], ABC):
+class LayerRuleObject(LayerSpecification[RuleApplier, Union[str, list[str]]], ABC):
     pass
 
 
@@ -269,9 +276,12 @@ class LayerBase(ABC):
         pass
 
 
-class LayerRuleBase(ABC):
+X = TypeVar("X", bound=BaseLayeredArchitecture)
+
+
+class LayerRuleBase(Generic[X]):
     """Entry point to defining a rule based on a layered architecture."""
 
     @abstractmethod
-    def based_on(self, architecture: BaseLayeredArchitecture) -> LayerBase:
+    def based_on(self, architecture: X) -> LayerBase:
         pass
