@@ -1,10 +1,16 @@
 from __future__ import annotations
 
+from collections.abc import Sequence
 from dataclasses import dataclass, replace
-from typing import Callable, List, Optional, Tuple, Union
+from typing import Callable
 
 from pytestarch import EvaluableArchitecture
-from pytestarch.eval_structure.evaluable_architecture import Module
+from pytestarch.eval_structure.evaluable_architecture import (
+    ModuleFilter,
+    ModuleNameFilter,
+    ModuleNameRegexFilter,
+    ParentModuleNameFilter,
+)
 from pytestarch.query_language.base_language import (
     BehaviorSpecification,
     DependencySpecification,
@@ -30,13 +36,13 @@ from pytestarch.utils.partial_match_to_regex_converter import (
 
 @dataclass
 class RuleConfiguration:
-    modules_to_check: Optional[List[Module]] = None
-    modules_to_check_against: Optional[List[Module]] = None
+    modules_to_check: Sequence[ModuleFilter] | None = None
+    modules_to_check_against: Sequence[ModuleFilter] | None = None
     should: bool = False
     should_only: bool = False
     should_not: bool = False
     except_present: bool = False
-    import_: bool = None
+    import_: bool | None = None
     rule_object_anything: bool = False
 
 
@@ -59,48 +65,52 @@ class Rule(
         ] = DefaultRuleMatcher,
     ) -> None:
         self._rule_matcher_class = rule_matcher_class
-        self._modules_to_check_to_be_specified_next = None
+        self._modules_to_check_to_be_specified_next: bool | None = None
         self._configuration = RuleConfiguration()
 
     @property
-    def rule_subjects(self) -> Optional[List[Module]]:
+    def rule_subjects(self) -> Sequence[ModuleFilter] | None:
         return self._configuration.modules_to_check
 
     def modules_that(self) -> RuleSubject:
         self._modules_to_check_to_be_specified_next = True
         return self
 
-    def are_sub_modules_of(
-        self, modules: Union[str, List[str]]
+    def are_sub_modules_of(  # type: ignore
+        self, modules: str | list[str]
     ) -> BehaviorSpecification:
-        self._set_modules(modules, lambda name: Module(parent_module=name))
+        self._set_modules(
+            modules, lambda name: ParentModuleNameFilter(parent_module=name)
+        )
         return self
 
-    def are_named(self, names: Union[str, List[str]]) -> BehaviorSpecification:
-        self._set_modules(names, lambda name: Module(name=name))
+    def are_named(self, names: str | list[str]) -> BehaviorSpecification:  # type: ignore
+        self._set_modules(names, lambda name: ModuleNameFilter(name=name))
         return self
 
     @deprecated
     def have_name_containing(
-        self, partial_names: Union[str, List[str]]
+        self, partial_names: str | list[str]
     ) -> BehaviorSpecification:
         self._set_modules(
             partial_names,
-            lambda name: Module(name=convert_partial_match_to_regex(name), regex=True),
+            lambda name: ModuleNameRegexFilter(
+                name=convert_partial_match_to_regex(name)
+            ),
         )
         return self
 
-    def have_name_matching(
+    def have_name_matching(  # type: ignore
         self,
         regex: str,
     ) -> BehaviorSpecification:
-        self._set_modules(regex, lambda name: Module(name=name, regex=True))
+        self._set_modules(regex, lambda name: ModuleNameRegexFilter(name=name))
         return self
 
     def _set_modules(
         self,
-        module_names: Union[str, List[str]],
-        create_module_fn: Callable[[str], Module],
+        module_names: str | list[str],
+        create_module_fn: Callable[[str], ModuleFilter],
     ) -> None:
         if self._modules_to_check_to_be_specified_next is None:
             raise ImproperlyConfigured("Specify a RuleSubject or RuleObject first.")
@@ -114,14 +124,18 @@ class Rule(
         else:
             self._configuration.modules_to_check_against = modules
 
-    def _add_modules(self, modules: List[Tuple[str, bool]]) -> BehaviorSpecification:
+    def _add_modules(self, modules: list[tuple[str, bool]]) -> BehaviorSpecification:
         module_names = []
         module_creation_fn = []
 
         for module, name_is_regex in modules:
             module_names.append(module)
             module_creation_fn.append(
-                lambda name: Module(name=name, regex=name_is_regex)
+                lambda name: (
+                    ModuleNameFilter(name=name)
+                    if not name_is_regex
+                    else ModuleNameRegexFilter(name=name)
+                )
             )
 
         self._append_modules(module_names, module_creation_fn)
@@ -129,20 +143,20 @@ class Rule(
 
     def _append_modules(
         self,
-        module_names: List[str],
-        create_module_fns: List[Callable[[str], Module]],
+        module_names: list[str],
+        create_module_fns: list[Callable[[str], ModuleFilter]],
     ) -> None:
         modules = [fn(n) for n, fn in zip(module_names, create_module_fns)]
         if self._modules_to_check_to_be_specified_next:
             if self._configuration.modules_to_check is None:
                 self._configuration.modules_to_check = []
 
-            self._configuration.modules_to_check.extend(modules)
+            self._configuration.modules_to_check.extend(modules)  # type: ignore
         else:
             if self._configuration.modules_to_check_against is None:
                 self._configuration.modules_to_check_against = []
 
-            self._configuration.modules_to_check_against.extend(modules)
+            self._configuration.modules_to_check_against.extend(modules)  # type: ignore
 
     def should(self) -> DependencySpecification:
         self._configuration.should = True
@@ -197,9 +211,9 @@ class Rule(
 
     def _prepare_rule_matcher(self) -> RuleMatcher:
         module_requirement = ModuleRequirement(
-            self._configuration.modules_to_check,
-            self._configuration.modules_to_check_against,
-            self._configuration.import_,
+            self._configuration.modules_to_check,  # type: ignore
+            self._configuration.modules_to_check_against,  # type: ignore
+            self._configuration.import_,  # type: ignore
         )
         behavior_requirement = BehaviorRequirement(
             self._configuration.should,
@@ -217,19 +231,19 @@ class Rule(
 
         subject_prefix = (
             "Sub modules of "
-            if self._configuration.modules_to_check[0].parent_module is not None
+            if self._configuration.modules_to_check[0].identifier_is_parent_module  # type: ignore
             else ""
         )
         if self._configuration.rule_object_anything:
             object_message = "anything "
         else:
-            object_message = f'modules that are {"named" if self._configuration.modules_to_check_against[0].name is not None else "sub modules of"} '
+            object_message = f'modules that are {"sub modules of" if self._configuration.modules_to_check_against[0].identifier_is_parent_module else "named"} '  # type: ignore
 
         combined_rule_subjects = self._combine_names(
-            self._configuration.modules_to_check
+            self._configuration.modules_to_check  # type: ignore
         )
         combined_rule_objects = self._combine_names(
-            self._configuration.modules_to_check_against
+            self._configuration.modules_to_check_against  # type: ignore
         )
         return (
             f'{subject_prefix}"{combined_rule_subjects}" '
@@ -240,13 +254,8 @@ class Rule(
             f'"{combined_rule_objects}".'
         )
 
-    def _combine_names(self, modules: list[Module]) -> str:
-        return ", ".join(
-            sorted(map(lambda module: self._get_module_name(module), modules))
-        )
-
-    def _get_module_name(self, module: Module) -> str:
-        return module.name if module.name is not None else module.parent_module
+    def _combine_names(self, modules: Sequence[ModuleFilter]) -> str:
+        return ", ".join(sorted(map(lambda module: module.identifier, modules)))
 
     def _assert_required_configuration_present(self) -> None:
         behavior_missing = not any(
@@ -320,21 +329,24 @@ class Rule(
     @classmethod
     def _get_modules_to_check_without_parent_and_submodule_combinations(
         cls, configuration: RuleConfiguration
-    ) -> Optional[List[Module]]:
+    ) -> Sequence[ModuleFilter] | None:
         # if modules_to_check contain a module and its submodule, this can throw off the breadth first search conducted
         # on the dependency graph - and also, this setup does not really make sense
         if configuration.modules_to_check is None:
             return None
 
         module_names = sorted(
-            map(lambda module: module.name, configuration.modules_to_check)
+            map(lambda m: m.identifier, configuration.modules_to_check)
         )
 
         result = []
         for module in configuration.modules_to_check:
             parent_module_found = False
             for module_name in module_names:
-                if module_name in module.name and module.name != module_name:
+                if (
+                    module_name in module.identifier
+                    and module.identifier != module_name
+                ):
                     parent_module_found = True
 
             if not parent_module_found:
