@@ -4,13 +4,15 @@ from __future__ import annotations
 
 from abc import ABC, abstractmethod
 from collections import defaultdict
+from collections.abc import Iterable
 from dataclasses import dataclass
-from typing import Dict, List, Optional, Set, Tuple
+from typing import cast
 
 from pytestarch.eval_structure.evaluable_architecture import (
+    Dependency,
+    Layer,
     LayerMapping,
     Module,
-    StrictDependency,
 )
 from pytestarch.rule_assessment.rule_check.rule_violations import RuleViolations
 
@@ -35,7 +37,7 @@ class RuleViolatedMessage:
 
 
 # (Import, negated, subject singular)
-PREFIX_MAPPING = defaultdict(str)
+PREFIX_MAPPING: defaultdict[tuple[bool, bool, bool], str] = defaultdict(str)
 PREFIX_MAPPING.update(
     {
         (False, False, True): "is ",
@@ -54,7 +56,7 @@ class RuleViolationMessageBaseGenerator(ABC):
 
     def create_rule_violation_messages(
         self, rule_violations: RuleViolations
-    ) -> List[str]:
+    ) -> list[str]:
         """Create a message about each rule violation.
 
         Args:
@@ -73,8 +75,8 @@ class RuleViolationMessageBaseGenerator(ABC):
 
     def _create_violation_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
-        messages = []
+    ) -> list[RuleViolatedMessage]:
+        messages: list[RuleViolatedMessage] = []
 
         self._extend(
             messages, self._create_should_import_violated_messages(rule_violations)
@@ -102,8 +104,8 @@ class RuleViolationMessageBaseGenerator(ABC):
 
     def _extend(
         self,
-        messages: List[RuleViolatedMessage],
-        new_messages: Optional[List[RuleViolatedMessage]],
+        messages: list[RuleViolatedMessage],
+        new_messages: list[RuleViolatedMessage] | None,
     ) -> None:
         if new_messages is not None:
             messages.extend(new_messages)
@@ -111,37 +113,37 @@ class RuleViolationMessageBaseGenerator(ABC):
     @abstractmethod
     def _create_should_import_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
+    ) -> list[RuleViolatedMessage]:
         pass
 
     @abstractmethod
     def _create_should_only_import_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
+    ) -> list[RuleViolatedMessage]:
         pass
 
     @abstractmethod
     def _create_should_not_import_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
+    ) -> list[RuleViolatedMessage]:
         pass
 
     @abstractmethod
     def _create_should_import_except_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
+    ) -> list[RuleViolatedMessage]:
         pass
 
     @abstractmethod
     def _create_should_only_import_except_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
+    ) -> list[RuleViolatedMessage]:
         pass
 
     @abstractmethod
     def _create_should_not_import_except_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
+    ) -> list[RuleViolatedMessage]:
         pass
 
 
@@ -159,15 +161,15 @@ class RuleViolationMessageGenerator(RuleViolationMessageBaseGenerator):
 
     def _create_should_import_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
+    ) -> list[RuleViolatedMessage]:
         return self._create_no_import_between_original_subject_and_objects_message(
             rule_violations.should_violations
         )
 
     def _create_no_import_between_original_subject_and_objects_message(
-        self, rule_violations: List[StrictDependency]
-    ) -> List[RuleViolatedMessage]:
-        messages = []
+        self, rule_violations: Iterable[Dependency]
+    ) -> list[RuleViolatedMessage]:
+        messages: list[RuleViolatedMessage] = []
 
         (
             rule_objects_for_rule_subject,
@@ -195,8 +197,8 @@ class RuleViolationMessageGenerator(RuleViolationMessageBaseGenerator):
 
     def _add_combined_rule_objects(
         self,
-        messages: List[RuleViolatedMessage],
-        rule_objects: List[str],
+        messages: list[RuleViolatedMessage],
+        rule_objects: list[str],
         rule_subject: str,
         rule_verb: str,
     ) -> None:
@@ -207,28 +209,28 @@ class RuleViolationMessageGenerator(RuleViolationMessageBaseGenerator):
             )
 
     def _get_violating_rule_subjects_and_objects(
-        self, rule_violation_dependency_names: List[StrictDependency]
-    ) -> Tuple[Dict[Module, List[Module]], Set[Module]]:
+        self, rule_violation_dependencies: Iterable[Dependency]
+    ) -> tuple[dict[Module, list[Module]], set[Module]]:
         violating_rule_subjects = set()
         rule_objects_for_rule_subject = defaultdict(list)
-        for rule_subject, rule_object in rule_violation_dependency_names:
+        for rule_subject, rule_object in rule_violation_dependencies:
             violating_rule_subjects.add(rule_subject)
             rule_objects_for_rule_subject[rule_subject].append(rule_object)
 
         return rule_objects_for_rule_subject, violating_rule_subjects
 
     def _convert_to_names(
-        self, violating_dependencies: List[StrictDependency]
-    ) -> List[Tuple[str, str]]:
+        self, violating_dependencies: list[Dependency]
+    ) -> list[tuple[str, str]]:
         return [
-            (dependency[0].name, dependency[1].name)
+            (dependency[0].identifier, dependency[1].identifier)
             for dependency in violating_dependencies
         ]
 
     def _create_should_only_import_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
-        messages = []
+    ) -> list[RuleViolatedMessage]:
+        messages: list[RuleViolatedMessage] = []
 
         self._extend(
             messages,
@@ -247,15 +249,15 @@ class RuleViolationMessageGenerator(RuleViolationMessageBaseGenerator):
 
     def _create_should_only_import_forbidden_import_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
+    ) -> list[RuleViolatedMessage]:
         return self._create_other_violating_dependencies_message(
             rule_violations.should_only_violations_by_forbidden_import
         )
 
     def _create_other_violating_dependencies_message(
         self,
-        violating_dependencies: List[StrictDependency],
-    ) -> List[RuleViolatedMessage]:
+        violating_dependencies: Iterable[Dependency],
+    ) -> list[RuleViolatedMessage]:
         # messages are always of the type "module x imports module y"
         messages = []
 
@@ -278,29 +280,29 @@ class RuleViolationMessageGenerator(RuleViolationMessageBaseGenerator):
 
     def _create_should_only_import_no_import_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
+    ) -> list[RuleViolatedMessage]:
         return self._create_no_import_between_original_subject_and_objects_message(
             rule_violations.should_only_violations_by_no_import
         )
 
     def _create_should_not_import_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
+    ) -> list[RuleViolatedMessage]:
         return self._create_other_violating_dependencies_message(
             rule_violations.should_not_violations
         )
 
     def _create_should_import_except_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
+    ) -> list[RuleViolatedMessage]:
         return self._create_no_import_other_than_between_original_subject_and_objects_message(
             rule_violations.should_except_violations
         )
 
     def _create_no_import_other_than_between_original_subject_and_objects_message(
-        self, rule_violations: List[StrictDependency]
-    ) -> List[RuleViolatedMessage]:
-        messages = []
+        self, rule_violations: Iterable[Dependency]
+    ) -> list[RuleViolatedMessage]:
+        messages: list[RuleViolatedMessage] = []
 
         (
             rule_objects_for_rule_subject,
@@ -330,8 +332,8 @@ class RuleViolationMessageGenerator(RuleViolationMessageBaseGenerator):
 
     def _add_combined_any_rule_objects(
         self,
-        messages: List[RuleViolatedMessage],
-        rule_objects: List[str],
+        messages: list[RuleViolatedMessage],
+        rule_objects: list[str],
         rule_subject: str,
         rule_verb: str,
         rule_object_type: str = ANY_MODULE_THAT_IS_NOT,
@@ -345,8 +347,8 @@ class RuleViolationMessageGenerator(RuleViolationMessageBaseGenerator):
 
     def _create_should_only_import_except_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
-        messages = []
+    ) -> list[RuleViolatedMessage]:
+        messages: list[RuleViolatedMessage] = []
 
         self._extend(
             messages,
@@ -365,46 +367,33 @@ class RuleViolationMessageGenerator(RuleViolationMessageBaseGenerator):
 
     def _create_should_only_import_except_forbidden_import_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
+    ) -> list[RuleViolatedMessage]:
         return self._create_other_violating_dependencies_message(
             rule_violations.should_only_except_violations_by_forbidden_import
         )
 
     def _create_should_only_import_except_no_import_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
+    ) -> list[RuleViolatedMessage]:
         return self._create_no_import_other_than_between_original_subject_and_objects_message(
             rule_violations.should_only_except_violations_by_no_import
         )
 
     def _create_should_not_import_except_violated_messages(
         self, rule_violations: RuleViolations
-    ) -> List[RuleViolatedMessage]:
+    ) -> list[RuleViolatedMessage]:
         return self._create_other_violating_dependencies_message(
             rule_violations.should_not_except_violations
         )
 
-    def _generate_rule_xbject_names_and_count(
-        self, original_rule_xbjects: List[Module]
-    ) -> Tuple[List[str], Dict[str, bool]]:
-        names = []
-        is_singular = {}
-
-        for rule_object in original_rule_xbjects:
-            module_name = self._get_module_name(rule_object)
-            names.append(module_name)
-            is_singular[module_name] = self._is_singular_module(rule_object)
-
-        return names, is_singular
-
     def _get_module_name(self, module: Module) -> str:
-        return module.name if module.name is not None else module.parent_module
+        return module.identifier
 
     def _is_singular_module(self, module: Module) -> bool:
-        return module.name is not None
+        return module.is_single_module
 
     def _get_verb_prefix(self, negated: bool, subject_singular: bool) -> str:
-        return PREFIX_MAPPING[(self._import_rule, negated, subject_singular)]
+        return PREFIX_MAPPING[(self._import_rule, negated, subject_singular)]  # type: ignore
 
     def _concatenate_verb(self, verb: str, prefix: str = "", suffix="") -> str:
         return f"{prefix}{verb}{suffix}"
@@ -430,8 +419,8 @@ class RuleViolationMessageGenerator(RuleViolationMessageBaseGenerator):
         return f'"{name}"'
 
     def _get_rule_subject_and_object_of_dependency(
-        self, dependency: StrictDependency
-    ) -> Tuple[str, str]:
+        self, dependency: Dependency
+    ) -> tuple[str, str]:
         rule_subject_name = self._get_module_name(dependency[0])
         rule_object_name = self._get_module_name(dependency[1])
 
@@ -483,9 +472,9 @@ class LayerRuleViolationMessageGenerator(RuleViolationMessageGenerator):
             return f" (layer {self._get_quoted_name(layer)})"
 
     def _create_no_import_between_original_subject_and_objects_message(
-        self, rule_violations: List[StrictDependency]
-    ) -> List[RuleViolatedMessage]:
-        messages = []
+        self, rule_violations: Iterable[Dependency]
+    ) -> list[RuleViolatedMessage]:
+        messages: list[RuleViolatedMessage] = []
 
         (
             rule_object_layers_for_rule_subject_layer,
@@ -518,14 +507,20 @@ class LayerRuleViolationMessageGenerator(RuleViolationMessageGenerator):
         return messages
 
     def _get_violating_rule_subject_and_objects_layers(
-        self, rule_violation_dependency_names: List[StrictDependency]
-    ) -> Tuple[Dict[str, Set[str]], Set[str]]:
+        self, rule_violation_dependencies: Iterable[Dependency]
+    ) -> tuple[defaultdict[str, set[str]], set[str]]:
         violating_rule_subject_layers = set()
         rule_object_layers_for_rule_subject_layer = defaultdict(set)
 
-        for rule_subject, rule_object in rule_violation_dependency_names:
-            rule_subject_layer = self._layer_mapping.get_layer(rule_subject)
-            rule_object_layer = self._layer_mapping.get_layer(rule_object)
+        for rule_subject, rule_object in rule_violation_dependencies:
+            rule_subject_layer = self._layer_mapping.get_layer_for_module_name(
+                rule_subject.identifier
+            )
+            rule_subject_layer = cast(Layer, rule_subject_layer)
+            rule_object_layer = self._layer_mapping.get_layer_for_module_name(
+                rule_object.identifier
+            )
+            rule_object_layer = cast(Layer, rule_object_layer)
 
             violating_rule_subject_layers.add(rule_subject_layer)
             rule_object_layers_for_rule_subject_layer[rule_subject_layer].add(
@@ -535,9 +530,9 @@ class LayerRuleViolationMessageGenerator(RuleViolationMessageGenerator):
         return rule_object_layers_for_rule_subject_layer, violating_rule_subject_layers
 
     def _create_no_import_other_than_between_original_subject_and_objects_message(
-        self, rule_violations: List[StrictDependency]
-    ) -> List[RuleViolatedMessage]:
-        messages = []
+        self, rule_violations: Iterable[Dependency]
+    ) -> list[RuleViolatedMessage]:
+        messages: list[RuleViolatedMessage] = []
 
         (
             rule_object_layers_for_rule_subject,
