@@ -2,13 +2,14 @@ from __future__ import annotations
 
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
-from typing import Any, Dict, List, Optional, Set
+from typing import Any
 
 from pytestarch.eval_structure.evaluable_architecture import (
+    Dependency,
     ExplicitlyRequestedDependenciesByBaseModules,
     NotExplicitlyRequestedDependenciesByBaseModule,
-    StrictDependency,
 )
+from pytestarch.eval_structure.utils import filter_to_module
 from pytestarch.rule_assessment.rule_check.behavior_requirement import (
     BehaviorRequirement,
 )
@@ -41,12 +42,12 @@ class RuleViolationBaseDetector(ABC):
 
     def get_rule_violation(
         self,
-        explicitly_requested_dependencies: Optional[
-            ExplicitlyRequestedDependenciesByBaseModules
-        ],
-        not_explicitly_requested_dependencies: Optional[
-            NotExplicitlyRequestedDependenciesByBaseModule
-        ],
+        explicitly_requested_dependencies: (
+            ExplicitlyRequestedDependenciesByBaseModules | None
+        ),
+        not_explicitly_requested_dependencies: (
+            NotExplicitlyRequestedDependenciesByBaseModule | None
+        ),
     ) -> RuleViolations:
         """Translate from the detected types of dependencies back to which behavior and dependency requirements are violated by them.
         Args:
@@ -96,85 +97,85 @@ class RuleViolationBaseDetector(ABC):
     def _should_not_requirement_violations(
         self,
         explicitly_requested_dependencies_should_not_be_present: bool,
-        explicitly_requested_dependencies: Optional[
-            ExplicitlyRequestedDependenciesByBaseModules
-        ],
-    ) -> Set[StrictDependency]:
+        explicitly_requested_dependencies: (
+            ExplicitlyRequestedDependenciesByBaseModules | None
+        ),
+    ) -> set[Dependency]:
         pass
 
     @abstractmethod
     def _should_requirement_violations(
         self,
         explicitly_requested_dependencies_should_be_present: bool,
-        explicitly_requested_dependencies: Optional[
-            ExplicitlyRequestedDependenciesByBaseModules
-        ],
-    ) -> Set[StrictDependency]:
+        explicitly_requested_dependencies: (
+            ExplicitlyRequestedDependenciesByBaseModules | None
+        ),
+    ) -> set[Dependency]:
         pass
 
     @abstractmethod
     def _should_only_requirement_violations_by_no_import(
         self,
         explicitly_requested_dependencies_and_no_other_should_be_present: bool,
-        explicitly_requested_dependencies: Optional[
-            ExplicitlyRequestedDependenciesByBaseModules
-        ],
-    ) -> Set[StrictDependency]:
+        explicitly_requested_dependencies: (
+            ExplicitlyRequestedDependenciesByBaseModules | None
+        ),
+    ) -> set[Dependency]:
         pass
 
     @abstractmethod
     def _should_only_requirement_violations_by_not_explicitly_requested_dependency(
         self,
         explicitly_requested_dependencies_and_no_other_should_be_present: bool,
-        not_explicitly_requested_dependencies: Optional[
-            NotExplicitlyRequestedDependenciesByBaseModule
-        ],
-    ) -> Set[StrictDependency]:
+        not_explicitly_requested_dependencies: (
+            NotExplicitlyRequestedDependenciesByBaseModule | None
+        ),
+    ) -> set[Dependency]:
         pass
 
     @abstractmethod
     def _should_except_requirement_violations(
         self,
         at_least_one_not_explicitly_requested_dependency_should_be_present: bool,
-        not_explicitly_requested_dependencies: Optional[
-            NotExplicitlyRequestedDependenciesByBaseModule
-        ],
-    ) -> Set[StrictDependency]:
+        not_explicitly_requested_dependencies: (
+            NotExplicitlyRequestedDependenciesByBaseModule | None
+        ),
+    ) -> set[Dependency]:
         pass
 
     @abstractmethod
     def _should_only_except_requirement_violations_due_to_no_other_imports(
         self,
         explicitly_requested_dependency_should_not_but_others_should_be_present: bool,
-        not_explicitly_requested_dependencies: Optional[
-            NotExplicitlyRequestedDependenciesByBaseModule
-        ],
-    ) -> Set[StrictDependency]:
+        not_explicitly_requested_dependencies: (
+            NotExplicitlyRequestedDependenciesByBaseModule | None
+        ),
+    ) -> set[Dependency]:
         pass
 
     @abstractmethod
     def _should_only_except_requirement_violations_due_to_explicit_dependency_present(
         self,
         explicitly_requested_dependency_should_not_but_others_should_be_present: bool,
-        explicitly_requested_dependencies: Optional[
-            ExplicitlyRequestedDependenciesByBaseModules
-        ],
-    ) -> Set[StrictDependency]:
+        explicitly_requested_dependencies: (
+            ExplicitlyRequestedDependenciesByBaseModules | None
+        ),
+    ) -> set[Dependency]:
         pass
 
     @abstractmethod
     def _should_not_except_requirement_violations(
         self,
         not_explicitly_requested_dependencies_should_not_be_present: bool,
-        not_explicitly_requested_dependencies: Optional[
-            NotExplicitlyRequestedDependenciesByBaseModule
-        ],
-    ) -> Set[StrictDependency]:
+        not_explicitly_requested_dependencies: (
+            NotExplicitlyRequestedDependenciesByBaseModule | None
+        ),
+    ) -> set[Dependency]:
         pass
 
     def _get_realised_dependencies(
-        self, explicitly_requested_dependencies: Dict[Any, List[StrictDependency]]
-    ) -> Set[StrictDependency]:
+        self, explicitly_requested_dependencies: dict[Any, list[Dependency]]
+    ) -> set[Dependency]:
         violating_dependencies = explicitly_requested_dependencies.values()
 
         violating_dependencies_in_user_specified_rule_subject_object_order = set()
@@ -193,8 +194,8 @@ class RuleViolationBaseDetector(ABC):
         return violating_dependencies_in_user_specified_rule_subject_object_order
 
     def _get_rule_subject_and_object_in_user_specified_order(
-        self, dependency: StrictDependency
-    ) -> StrictDependency:
+        self, dependency: Dependency
+    ) -> Dependency:
         if self._module_requirement.rule_specified_with_importer_as_rule_subject:
             return dependency
 
@@ -228,6 +229,12 @@ class RuleViolationBaseDetector(ABC):
             ),
         )
 
+    def _get_importee_modules_as_specified_by_user(self):
+        return [
+            filter_to_module(filter)
+            for filter in self._module_requirement.importees_as_specified_by_user
+        ]
+
 
 class RuleViolationDetector(RuleViolationBaseDetector):
     """Based on the behavior requirement and the dependencies that have actually been (not) found in the graph,
@@ -239,21 +246,21 @@ class RuleViolationDetector(RuleViolationBaseDetector):
     NotExplicitlyRequestedDependenciesByBaseModule on the other hand are structured like this:
     {Module for which not explicitly requested dependencies (either from or to this module) were found: [list of such dependencies]}.
 
-    The output is a rule violation object which for each type of rule contains a list of (rule subject, rule object) which violate the rule. These are ordered according
+    The output is a rule violation object which for each type of rule contains a list of (rule subject, rule object) modules which violate the rule. These are ordered according
     to the order the user used when specifying the rule, i.e. the rule subject is the original rule subject no matter if the rule is an import
     or be imported rule.
 
-    If multiple rule subjects have been specified, the rule needs to apply to each and every one of them in order not
+    If multiple rule subjects have been specified, the rule needs to apply to each one of them in order not
     to count as being violated.
     """
 
     def _should_not_requirement_violations(
         self,
         explicitly_requested_dependencies_should_not_be_present: bool,
-        explicitly_requested_dependencies: Optional[
-            ExplicitlyRequestedDependenciesByBaseModules
-        ],
-    ) -> Set[StrictDependency]:
+        explicitly_requested_dependencies: (
+            ExplicitlyRequestedDependenciesByBaseModules | None
+        ),
+    ) -> set[Dependency]:
         if (
             explicitly_requested_dependencies is None
             or not explicitly_requested_dependencies_should_not_be_present
@@ -265,10 +272,10 @@ class RuleViolationDetector(RuleViolationBaseDetector):
     def _should_requirement_violations(
         self,
         explicitly_requested_dependencies_should_be_present: bool,
-        explicitly_requested_dependencies: Optional[
-            ExplicitlyRequestedDependenciesByBaseModules
-        ],
-    ) -> Set[StrictDependency]:
+        explicitly_requested_dependencies: (
+            ExplicitlyRequestedDependenciesByBaseModules | None
+        ),
+    ) -> set[Dependency]:
         if (
             explicitly_requested_dependencies is None
             or not explicitly_requested_dependencies_should_be_present
@@ -282,10 +289,10 @@ class RuleViolationDetector(RuleViolationBaseDetector):
     def _should_only_requirement_violations_by_no_import(
         self,
         explicitly_requested_dependencies_and_no_other_should_be_present: bool,
-        explicitly_requested_dependencies: Optional[
-            ExplicitlyRequestedDependenciesByBaseModules
-        ],
-    ) -> Set[StrictDependency]:
+        explicitly_requested_dependencies: (
+            ExplicitlyRequestedDependenciesByBaseModules | None
+        ),
+    ) -> set[Dependency]:
         if (
             explicitly_requested_dependencies is None
             or not explicitly_requested_dependencies_and_no_other_should_be_present
@@ -299,10 +306,10 @@ class RuleViolationDetector(RuleViolationBaseDetector):
     def _should_only_requirement_violations_by_not_explicitly_requested_dependency(
         self,
         explicitly_requested_dependencies_and_no_other_should_be_present: bool,
-        not_explicitly_requested_dependencies: Optional[
-            NotExplicitlyRequestedDependenciesByBaseModule
-        ],
-    ) -> Set[StrictDependency]:
+        not_explicitly_requested_dependencies: (
+            NotExplicitlyRequestedDependenciesByBaseModule | None
+        ),
+    ) -> set[Dependency]:
         if (
             not_explicitly_requested_dependencies is None
             or not explicitly_requested_dependencies_and_no_other_should_be_present
@@ -314,10 +321,10 @@ class RuleViolationDetector(RuleViolationBaseDetector):
     def _should_except_requirement_violations(
         self,
         at_least_one_not_explicitly_requested_dependency_should_be_present: bool,
-        not_explicitly_requested_dependencies: Optional[
-            NotExplicitlyRequestedDependenciesByBaseModule
-        ],
-    ) -> Set[StrictDependency]:
+        not_explicitly_requested_dependencies: (
+            NotExplicitlyRequestedDependenciesByBaseModule | None
+        ),
+    ) -> set[Dependency]:
         if (
             not_explicitly_requested_dependencies is None
             or not at_least_one_not_explicitly_requested_dependency_should_be_present
@@ -331,10 +338,10 @@ class RuleViolationDetector(RuleViolationBaseDetector):
     def _should_only_except_requirement_violations_due_to_no_other_imports(
         self,
         explicitly_requested_dependency_should_not_but_others_should_be_present: bool,
-        not_explicitly_requested_dependencies: Optional[
-            NotExplicitlyRequestedDependenciesByBaseModule
-        ],
-    ) -> Set[StrictDependency]:
+        not_explicitly_requested_dependencies: (
+            NotExplicitlyRequestedDependenciesByBaseModule | None
+        ),
+    ) -> set[Dependency]:
         if (
             not_explicitly_requested_dependencies is None
             or not explicitly_requested_dependency_should_not_but_others_should_be_present
@@ -348,10 +355,10 @@ class RuleViolationDetector(RuleViolationBaseDetector):
     def _should_only_except_requirement_violations_due_to_explicit_dependency_present(
         self,
         explicitly_requested_dependency_should_not_but_others_should_be_present: bool,
-        explicitly_requested_dependencies: Optional[
-            ExplicitlyRequestedDependenciesByBaseModules
-        ],
-    ) -> Set[StrictDependency]:
+        explicitly_requested_dependencies: (
+            ExplicitlyRequestedDependenciesByBaseModules | None
+        ),
+    ) -> set[Dependency]:
         if (
             explicitly_requested_dependencies is None
             or not explicitly_requested_dependency_should_not_but_others_should_be_present
@@ -363,10 +370,10 @@ class RuleViolationDetector(RuleViolationBaseDetector):
     def _should_not_except_requirement_violations(
         self,
         not_explicitly_requested_dependencies_should_not_be_present: bool,
-        not_explicitly_requested_dependencies: Optional[
-            NotExplicitlyRequestedDependenciesByBaseModule
-        ],
-    ) -> Set[StrictDependency]:
+        not_explicitly_requested_dependencies: (
+            NotExplicitlyRequestedDependenciesByBaseModule | None
+        ),
+    ) -> set[Dependency]:
         if (
             not_explicitly_requested_dependencies is None
             or not not_explicitly_requested_dependencies_should_not_be_present
@@ -376,8 +383,8 @@ class RuleViolationDetector(RuleViolationBaseDetector):
         return self._get_realised_dependencies(not_explicitly_requested_dependencies)
 
     def _get_abstract_dependencies_without_realisations(
-        self, explicitly_requested_dependencies: Dict[Any, List[StrictDependency]]
-    ) -> Set[StrictDependency]:
+        self, explicitly_requested_dependencies: dict[Any, list[Dependency]]
+    ) -> set[Dependency]:
         violating_dependencies = [
             abstract_dependency
             for abstract_dependency, concrete_dependency in explicitly_requested_dependencies.items()
@@ -391,7 +398,7 @@ class RuleViolationDetector(RuleViolationBaseDetector):
     def _get_missing_dependencies_in_user_specified_order(
         self,
         not_explicitly_requested_dependencies: NotExplicitlyRequestedDependenciesByBaseModule,
-    ) -> Set[StrictDependency]:
+    ) -> set[Dependency]:
         dependencies = []
 
         for (
@@ -402,16 +409,12 @@ class RuleViolationDetector(RuleViolationBaseDetector):
                 continue
 
             if self._module_requirement.rule_specified_with_importer_as_rule_subject:
-                for (
-                    other_module
-                ) in self._module_requirement.importees_as_specified_by_user:
+                for other_module in self._get_importee_modules_as_specified_by_user():
                     dependencies.append(
                         (module_with_missing_dependencies, other_module)
                     )
             else:
-                for (
-                    other_module
-                ) in self._module_requirement.importees_as_specified_by_user:
+                for other_module in self._get_importee_modules_as_specified_by_user():
                     dependencies.append(
                         (other_module, module_with_missing_dependencies)
                     )
